use chrono::prelude::*;
use diesel::insert_into;
use diesel::prelude::*;

use crate::schema::backup;

#[derive(Queryable)]
pub struct Backup {
    pub uuid: String,
    pub filename: String,
    pub contents: Vec<u8>,
    pub creation_instant: NaiveDateTime,
    pub contest_id: i32,
    pub user_id: i32,
}

#[derive(Insertable)]
#[diesel(table_name = backup)]
pub struct NewBackup {
    pub uuid: String,
    pub filename: String,
    pub contents: Vec<u8>,
    pub creation_instant: NaiveDateTime,
    pub contest_id: i32,
    pub user_id: i32,
}

pub fn insert_backup(connection: &mut PgConnection, new_backup: NewBackup) -> QueryResult<()> {
    insert_into(backup::table)
        .values(new_backup)
        .execute(connection)?;
    Ok(())
}

pub fn get_backups_by_contest_user(
    connection: &mut PgConnection,
    user_id: i32,
    contest_id: i32,
) -> QueryResult<Vec<Backup>> {
    backup::table
        .filter(backup::user_id.eq(user_id))
        .filter(backup::contest_id.eq(contest_id))
        .load(connection)
}

pub fn get_backup_by_uuid(connection: &mut PgConnection, uuid: String) -> QueryResult<Backup> {
    backup::table
        .filter(backup::uuid.eq(uuid))
        .first(connection)
}
