use chrono::prelude::*;
use diesel::insert_into;
use diesel::prelude::*;

use crate::schema::repository;

#[derive(Queryable)]
pub struct Repository {
    pub id: i32,
    pub name: String,
    pub path: String,
    pub remote_url: String,
    pub creation_user_id: i32,
    pub creation_instant: NaiveDateTime,
}

#[derive(Insertable)]
#[diesel(table_name = repository)]
pub struct NewRepository {
    pub name: String,
    pub path: String,
    pub remote_url: String,
    pub creation_user_id: i32,
    pub creation_instant: NaiveDateTime,
}

pub fn insert_repository(
    connection: &mut PgConnection,
    new_repository: NewRepository,
) -> QueryResult<Repository> {
    insert_into(repository::table)
        .values(new_repository)
        .execute(connection)?;
    repository::table
        .order(repository::id.desc())
        .first(connection)
}

pub fn get_repositories(connection: &mut PgConnection) -> QueryResult<Vec<Repository>> {
    repository::table
        .order(repository::creation_instant.desc())
        .load(connection)
}

pub fn get_repository_by_id(connection: &mut PgConnection, id: i32) -> QueryResult<Repository> {
    repository::table
        .filter(repository::id.eq(id))
        .first(connection)
}
