use chrono::Local;
use serde_with::serde_as;
use serde_with::NoneAsEmptyString;
use uuid::Uuid;

use crate::models::clarification::NewClarification;
use crate::models::{clarification, contest};
use crate::pages::assert_contest_not_started;
use crate::pages::prelude::*;

#[serde_as]
#[derive(Serialize, Deserialize)]
struct CreateClarificationForm {
    question: String,
    #[serde_as(as = "NoneAsEmptyString")]
    contest_problem_id: Option<i32>,
}

#[post("/contests/{id}/clarifications/")]
async fn create_clarification(
    identity: Identity,
    pool: Data<DbPool>,
    form: Form<CreateClarificationForm>,
    path: Path<(i32,)>,
    request: HttpRequest,
) -> PageResult {
    let logged_user = require_identity(&identity)?;
    let (contest_id,) = path.into_inner();
    let mut connection = pool.get()?;

    let contest = contest::get_contest_by_id(&mut connection, contest_id)?;
    assert_contest_not_started(&logged_user, &contest)?;

    let uuid = Uuid::new_v4();
    clarification::insert_clarification(
        &mut connection,
        NewClarification {
            uuid: uuid.to_string(),
            question: form.question.clone(),
            question_instant: Local::now().naive_utc(),
            answer_to_all: false,
            contest_problem_id: form.contest_problem_id,
            contest_id: contest.id,
            user_id: logged_user.id,
        },
    )?;

    Ok(redirect_to_referer(
        "Clarificação criada com sucesso".into(),
        &request,
    ))
}
