use crate::models::backup;
use crate::pages::prelude::*;
use actix_web::{http::header::ContentType, HttpResponse};

#[get("/backups/{uuid}/contents")]
async fn get_backup_contents(
    identity: Identity,
    pool: Data<DbPool>,
    path: Path<(String,)>,
) -> PageResult {
    let logged_user = require_identity(&identity)?;
    let (backup_uuid,) = path.into_inner();
    let mut connection = pool.get()?;

    let backup = backup::get_backup_by_uuid(&mut connection, backup_uuid)?;

    if backup.user_id != logged_user.id && !logged_user.is_admin {
        return Err(PageError::Forbidden(
            "Não é possível acessar um backup de outro usuário".into(),
        ));
    }

    Ok(HttpResponse::Ok()
        .content_type(ContentType::octet_stream())
        .body(backup.contents))
}
