use crate::models::submission;
use crate::pages::prelude::*;
use crate::pages::{
    assert_not_single_contest, get_formatted_contests, get_formatted_submissions, FormattedContest,
    FormattedSubmission,
};

#[get("/")]
async fn get_main(
    base: BaseContext,
    identity: Identity,
    pool: Data<DbPool>,
    hb: Data<Handlebars<'_>>,
    tz: Data<Tz>,
) -> PageResult {
    let logged_user = require_identity(&identity)?;
    if let Some(redirect) = assert_not_single_contest(&logged_user) {
        return redirect;
    }

    #[derive(Serialize)]
    struct Context {
        base: BaseContext,
        contests: Vec<FormattedContest>,
        submissions: Vec<FormattedSubmission>,
    }

    let mut connection = pool.get()?;
    let submissions = if !logged_user.is_admin {
        submission::get_submissions_user(&mut connection, logged_user.id)?
    } else {
        submission::get_submissions(&mut connection)?
    };

    render(
        &hb,
        "main",
        &Context {
            base,
            contests: get_formatted_contests(&mut connection, Some(logged_user.id), &tz)?,
            submissions: if logged_user.is_admin {
                get_formatted_submissions(&tz, &submissions)
            } else {
                vec![]
            },
        },
    )
}
