use crate::models::printing_task;
use crate::pages::prelude::*;
use actix_web::{http::header::ContentType, HttpResponse};

#[get("/printing-tasks/{uuid}/contents")]
async fn get_printing_task_contents(
    identity: Identity,
    pool: Data<DbPool>,
    path: Path<(String,)>,
) -> PageResult {
    let logged_user = require_identity(&identity)?;
    let (printing_task_uuid,) = path.into_inner();
    let mut connection = pool.get()?;

    let (printing_task, user) =
        printing_task::get_printing_task_by_uuid(&mut connection, printing_task_uuid)?;

    if user.id != logged_user.id && !logged_user.is_admin {
        return Err(PageError::Forbidden(
            "Não é possível acessar uma tarefa de impressão de outro usuário".into(),
        ));
    }

    let user_full_name = user.full_name.unwrap_or("".into());
    Ok(HttpResponse::Ok()
        .content_type(ContentType::plaintext())
        .body(format!(
            r#"///// {} -- {} -- {}
///// {} -- {} -- {}
///// {} -- {} -- {}


{}

///// {} -- {} -- {}
///// {} -- {} -- {}
///// {} -- {} -- {}
"#,
            user.name,
            user_full_name,
            user.name,
            user.name,
            user_full_name,
            user.name,
            user.name,
            user_full_name,
            user.name,
            printing_task.contents,
            user.name,
            user_full_name,
            user.name,
            user.name,
            user_full_name,
            user.name,
            user.name,
            user_full_name,
            user.name
        )))
}
