use std::path::PathBuf;

use actix_files::NamedFile;
use actix_web::http::header::ContentDisposition;
use actix_web::http::header::DispositionParam;

use crate::models::{contest, problem};
use crate::pages::assert_contest_not_started;
use crate::pages::prelude::*;

#[get("/problems/{id}/assets/{filename}")]
async fn get_problem_by_id_assets(
    identity: Identity,
    pool: Data<DbPool>,
    path: Path<(i32, String)>,
) -> Result<NamedFile, PageError> {
    let logged_user = require_identity(&identity)?;
    let (problem_id, asset_filename) = path.into_inner();
    let mut connection = pool.get()?;
    let problem = problem::get_problem_by_contest_id_metadata(&mut connection, problem_id)?;
    let contest = contest::get_contest_by_id(&mut connection, problem.contest_id)?;
    assert_contest_not_started(&logged_user, &contest)?;
    let file_path_prefix = PathBuf::from("/data/").join(problem.id).join("statement");
    let file_path = file_path_prefix.join(asset_filename).canonicalize()?;
    if !file_path.starts_with(file_path_prefix) {
        return Err(PageError::Forbidden(
            "Caminho para o problema deve estar dentro do repositório".into(),
        ));
    }
    let extension = file_path.extension().unwrap().to_str().unwrap();
    let mut named_file = NamedFile::open(file_path.clone())?;
    let content_disposition = named_file.content_disposition().clone();
    named_file = named_file.set_content_disposition(ContentDisposition {
        disposition: content_disposition.disposition,
        parameters: vec![DispositionParam::Filename(problem.label + "." + extension)],
    });
    println!("{}", named_file.content_disposition());
    Ok(named_file)
}
