use crate::models::repository;
use crate::pages::prelude::*;

#[get("/repositories/")]
async fn get_repositories(
    base: BaseContext,
    identity: Identity,
    pool: Data<DbPool>,
    hb: Data<Handlebars<'_>>,
    tz: Data<Tz>,
) -> PageResult {
    let logged_user = require_identity(&identity)?;
    if !logged_user.is_admin {
        return Err(PageError::Forbidden(
            "Apenas administradores podem fazer isso".into(),
        ));
    }

    #[derive(Serialize)]
    struct FormattedRepository {
        pub id: i32,
        pub name: String,
        pub remote_url: String,
        pub creation_instant: String,
    }

    #[derive(Serialize)]
    struct Context {
        base: BaseContext,
        repositories: Vec<FormattedRepository>,
    }

    let mut connection = pool.get()?;
    render(
        &hb,
        "repositories",
        &Context {
            base,
            repositories: repository::get_repositories(&mut connection)?
                .iter()
                .map(|r| FormattedRepository {
                    id: r.id,
                    name: r.name.clone(),
                    remote_url: r.remote_url.clone(),
                    creation_instant: format_utc_date_time(&tz, r.creation_instant),
                })
                .collect(),
        },
    )
}
