use crate::models::{backup, contest, submission};
use crate::pages::prelude::*;
use crate::pages::{
    assert_single_contest_not_started, get_formatted_contest,
    get_formatted_submissions_with_blind_ascending, FormattedContest, FormattedSubmission,
};

#[get("/single-contest/{id}/backup")]
async fn get_single_contest_backup(
    base: BaseContext,
    identity: Identity,
    pool: Data<DbPool>,
    hb: Data<Handlebars<'_>>,
    path: Path<(i32,)>,
    tz: Data<Tz>,
) -> PageResult {
    let logged_user = require_identity(&identity)?;
    let (contest_id,) = path.into_inner();

    #[derive(Serialize)]
    struct FormattedBackup {
        pub uuid: String,
        pub creation_instant: String,
        pub filename: String,
    }

    #[derive(Serialize)]
    struct Context {
        base: BaseContext,
        contest: FormattedContest,
        backups: Vec<FormattedBackup>,
        submissions: Vec<FormattedSubmission>,
    }

    let mut connection = pool.get()?;
    let contest = contest::get_contest_by_id(&mut connection, contest_id)?;
    if let Some(redirect) = assert_single_contest_not_started(&logged_user, &contest) {
        return redirect;
    }
    let submissions =
        submission::get_submissions_user_by_contest(&mut connection, logged_user.id, contest_id)?;
    let backups = backup::get_backups_by_contest_user(&mut connection, logged_user.id, contest_id)?;

    render(
        &hb,
        "single_backup",
        &Context {
            base,
            contest: get_formatted_contest(&tz, &contest),
            backups: backups
                .into_iter()
                .map(|b| FormattedBackup {
                    uuid: b.uuid,
                    creation_instant: format_utc_date_time(&tz, b.creation_instant),
                    filename: b.filename,
                })
                .collect(),
            submissions: get_formatted_submissions_with_blind_ascending(
                &tz,
                &contest,
                &submissions,
            ),
        },
    )
}
