use crate::models::{contest, printing_task, submission};
use crate::pages::prelude::*;
use crate::pages::{
    assert_single_contest_not_started, get_formatted_contest,
    get_formatted_submissions_with_blind_ascending, FormattedContest, FormattedSubmission,
};

#[get("/single-contest/{id}/tasks")]
async fn get_single_contest_tasks(
    base: BaseContext,
    identity: Identity,
    pool: Data<DbPool>,
    hb: Data<Handlebars<'_>>,
    path: Path<(i32,)>,
    tz: Data<Tz>,
) -> PageResult {
    let logged_user = require_identity(&identity)?;
    let (contest_id,) = path.into_inner();

    #[derive(Serialize)]
    struct FormattedPrintingTask {
        pub uuid: String,
        pub creation_instant: String,
        pub status: String,
        pub done_instant: Option<String>,
    }

    #[derive(Serialize)]
    struct Context {
        base: BaseContext,
        contest: FormattedContest,
        printing_tasks: Vec<FormattedPrintingTask>,
        submissions: Vec<FormattedSubmission>,
    }

    let mut connection = pool.get()?;
    let contest = contest::get_contest_by_id(&mut connection, contest_id)?;
    if let Some(redirect) = assert_single_contest_not_started(&logged_user, &contest) {
        return redirect;
    }
    let submissions =
        submission::get_submissions_user_by_contest(&mut connection, logged_user.id, contest_id)?;
    let printing_tasks = printing_task::get_printing_tasks_by_contest_user(
        &mut connection,
        logged_user.id,
        contest_id,
    )?;

    render(
        &hb,
        "single_tasks",
        &Context {
            base,
            contest: get_formatted_contest(&tz, &contest),
            printing_tasks: printing_tasks
                .into_iter()
                .map(|p| FormattedPrintingTask {
                    uuid: p.uuid,
                    status: p.status,
                    creation_instant: format_utc_date_time(&tz, p.creation_instant),
                    done_instant: p.done_instant.map(|i| format_utc_date_time(&tz, i)),
                })
                .collect(),
            submissions: get_formatted_submissions_with_blind_ascending(
                &tz,
                &contest,
                &submissions,
            ),
        },
    )
}
