use crate::models::contest;
use crate::pages::get_formatted_contest;
use crate::pages::prelude::*;
use actix_web::http::header;
use actix_web::http::header::HeaderValue;
use chrono::Local;
use std::env;

#[get("/single-contest/{id}/waiting")]
pub async fn get_single_contest_waiting(
    base: BaseContext,
    identity: Identity,
    pool: Data<DbPool>,
    hb: Data<Handlebars<'_>>,
    path: Path<(i32,)>,
    tz: Data<Tz>,
) -> PageResult {
    require_identity(&identity)?;
    let (contest_id,) = path.into_inner();
    let mut connection = pool.get()?;
    let contest = contest::get_contest_by_id(&mut connection, contest_id)?;

    if contest
        .start_instant
        .map_or(true, |s| s < Local::now().naive_utc())
    {
        return Ok(HttpResponse::SeeOther()
            .append_header((
                header::LOCATION,
                HeaderValue::from_str(&format!(
                    "{}/single-contest/{}/problems",
                    env::var("BASE_URL").expect("BASE_URL environment variable is not set"),
                    contest_id
                ))
                .unwrap(),
            ))
            .finish());
    }

    let start_instant = contest.start_instant.unwrap();
    let f = get_formatted_contest(&tz, &contest);

    #[derive(Serialize)]
    struct FormattedContest {
        start_instant: String,
        id: i32,
        contest_status: String,
    }

    #[derive(Serialize)]
    struct Context {
        base: BaseContext,
        contest: FormattedContest,
    }

    render(
        &hb,
        "single_waiting",
        &Context {
            base,
            contest: FormattedContest {
                id: contest_id,
                start_instant: start_instant.format("%Y-%m-%dT%H:%M:%SZ").to_string(),
                contest_status: f.contest_status,
            },
        },
    )
}
