use crate::models::submission;
use crate::pages::prelude::*;
use actix_web::{http::header::ContentType, HttpResponse};

#[get("/submissions/{uuid}/source")]
async fn get_submission_source(
    identity: Identity,
    pool: Data<DbPool>,
    path: Path<(String,)>,
) -> PageResult {
    let logged_user = require_identity(&identity)?;
    let (submission_uuid,) = path.into_inner();
    let mut connection = pool.get()?;

    let (submission, user, _, _) =
        submission::get_submission_by_uuid(&mut connection, submission_uuid)?;

    if user.id != logged_user.id && !logged_user.is_admin {
        return Err(PageError::Forbidden(
            "Não é possível acessar uma submissão de outro usuário".into(),
        ));
    }

    Ok(HttpResponse::Ok()
        .content_type(ContentType::plaintext())
        .body(submission.source_text))
}
