use submission::{ContestProblem, Submission};

use crate::models::submission;
use crate::pages::prelude::*;
use crate::pages::{assert_not_single_contest, get_formatted_submissions, FormattedSubmission};
use crate::user::User;

pub fn render_submissions(
    base: BaseContext,
    hb: &Handlebars<'_>,
    tz: &Tz,
    submissions: Vec<(Submission, ContestProblem, User)>,
) -> PageResult {
    #[derive(Serialize)]
    struct Context {
        base: BaseContext,
        submissions: Vec<FormattedSubmission>,
    }
    render(
        hb,
        "submissions",
        &Context {
            base,
            submissions: get_formatted_submissions(tz, &submissions),
        },
    )
}

#[get("/submissions/")]
async fn get_submissions(
    base: BaseContext,
    identity: Identity,
    pool: Data<DbPool>,
    hb: Data<Handlebars<'_>>,
    tz: Data<Tz>,
) -> PageResult {
    let logged_user = require_identity(&identity)?;
    if !logged_user.is_admin {
        return Err(PageError::Forbidden(
            "Apenas administradores podem fazer isso".into(),
        ));
    }
    if let Some(redirect) = assert_not_single_contest(&logged_user) {
        return redirect;
    }
    let mut connection = pool.get()?;
    render_submissions(
        base,
        &hb,
        &tz,
        submission::get_submissions(&mut connection)?,
    )
}
