use crate::models::submission;
use crate::pages::get_submissions::render_submissions;
use crate::pages::prelude::*;

#[get("/submissions/contests/{id}")]
async fn get_submissions_by_contest_id(
    base: BaseContext,
    identity: Identity,
    pool: Data<DbPool>,
    hb: Data<Handlebars<'_>>,
    path: Path<(i32,)>,
    tz: Data<Tz>,
) -> PageResult {
    let logged_user = require_identity(&identity)?;
    if !logged_user.is_admin {
        return Err(PageError::Forbidden(
            "Apenas administradores podem fazer isso".into(),
        ));
    }
    let (contest_id,) = path.into_inner();
    let mut connection = pool.get()?;

    render_submissions(
        base,
        &hb,
        &tz,
        submission::get_submissions_by_contest(&mut connection, contest_id)?,
    )
}
