use chrono::Local;

use crate::models::clarification;
use crate::pages::prelude::*;

#[derive(Deserialize)]
struct PatchClarificationForm {
    answer: String,
    answer_to_all: Option<String>,
}

#[patch("/clarifications/{uuid}")]
pub async fn patch_clarification(
    identity: Identity,
    pool: Data<DbPool>,
    form: Form<PatchClarificationForm>,
    path: Path<(String,)>,
    request: HttpRequest,
) -> PageResult {
    let logged_user = require_identity(&identity)?;
    if !logged_user.is_admin {
        return Err(PageError::Forbidden(
            "Apenas administradores podem fazer isso".into(),
        ));
    }
    let (clarification_uuid,) = path.into_inner();
    let mut connection = pool.get()?;

    clarification::update_clarification(
        &mut connection,
        clarification_uuid,
        form.answer.clone(),
        form.answer_to_all.as_ref().map(|e| e == "on").unwrap_or(false),
        logged_user.id,
        Local::now().naive_utc(),
    )?;

    Ok(redirect_to_referer(
        "Clarificação respondida com sucesso".into(),
        &request,
    ))
}
