use chrono::NaiveDateTime;

use crate::models::contest;
use crate::pages::prelude::*;

#[derive(Deserialize)]
struct PatchContestForm {
    start_instant: Option<String>,
    end_instant: Option<String>,
    frozen_scoreboard_instant: Option<String>,
    blind_scoreboard_instant: Option<String>,
}

#[patch("/contests/{id}")]
pub async fn patch_contest(
    identity: Identity,
    pool: Data<DbPool>,
    form: Form<PatchContestForm>,
    path: Path<(i32,)>,
    tz: Data<Tz>,
    request: HttpRequest,
) -> PageResult {
    let logged_user = require_identity(&identity)?;
    if !logged_user.is_admin {
        return Err(PageError::Forbidden(
            "Apenas administradores podem fazer isso".into(),
        ));
    }
    let (contest_id,) = path.into_inner();
    let mut connection = pool.get()?;

    contest::update_contest(
        &mut connection,
        contest_id,
        form.start_instant
            .as_ref()
            .and_then(|s| NaiveDateTime::parse_from_str(&s, "%Y-%m-%d %H:%M:%S").ok())
            .and_then(|d| d.and_local_timezone(**tz).single())
            .map(|d| d.naive_utc()),
        form.end_instant
            .as_ref()
            .and_then(|s| NaiveDateTime::parse_from_str(&s, "%Y-%m-%d %H:%M:%S").ok())
            .and_then(|d| d.and_local_timezone(**tz).single())
            .map(|d| d.naive_utc()),
        form.frozen_scoreboard_instant
            .as_ref()
            .and_then(|s| NaiveDateTime::parse_from_str(&s, "%Y-%m-%d %H:%M:%S").ok())
            .and_then(|d| d.and_local_timezone(**tz).single())
            .map(|d| d.naive_utc()),
        form.blind_scoreboard_instant
            .as_ref()
            .and_then(|s| NaiveDateTime::parse_from_str(&s, "%Y-%m-%d %H:%M:%S").ok())
            .and_then(|d| d.and_local_timezone(**tz).single())
            .map(|d| d.naive_utc()),
    )?;

    Ok(redirect_to_referer(
        "Competição atualizada com sucesso".into(),
        &request,
    ))
}
