use serde_with::serde_as;
use serde_with::NoneAsEmptyString;

use crate::models::contest;
use crate::pages::prelude::*;

#[serde_as]
#[derive(Deserialize)]
struct PatchContestForm {
    #[serde_as(as = "NoneAsEmptyString")]
    balloon_color: Option<String>,
}

#[patch("/contests/{id}/problems/{contest_problem_id}")]
pub async fn patch_contest_problem(
    identity: Identity,
    pool: Data<DbPool>,
    form: Form<PatchContestForm>,
    path: Path<(i32, i32)>,
    request: HttpRequest,
) -> PageResult {
    let logged_user = require_identity(&identity)?;
    if !logged_user.is_admin {
        return Err(PageError::Forbidden(
            "Apenas administradores podem fazer isso".into(),
        ));
    }
    let (_, contest_problem_id) = path.into_inner();
    let mut connection = pool.get()?;

    contest::update_contest_problem_balloon_color(
        &mut connection,
        contest_problem_id,
        form.balloon_color.clone(),
    )?;

    Ok(redirect_to_referer(
        "Problema atualizado com sucesso".into(),
        &request,
    ))
}
