use chrono::Local;

use crate::models::printing_task;
use crate::pages::prelude::*;

#[post("/printing-tasks/{uuid}/done")]
pub async fn post_printing_task_done(
    identity: Identity,
    pool: Data<DbPool>,
    path: Path<(String,)>,
    request: HttpRequest,
) -> PageResult {
    let logged_user = require_identity(&identity)?;
    if !logged_user.is_admin {
        return Err(PageError::Forbidden(
            "Apenas administradores podem fazer isso".into(),
        ));
    }
    let (printing_task_uuid,) = path.into_inner();
    let mut connection = pool.get()?;

    printing_task::update_printing_task(
        &mut connection,
        printing_task_uuid,
        Local::now().naive_utc(),
    )?;

    Ok(redirect_to_referer(
        "Tarefa fechada com sucesso".into(),
        &request,
    ))
}
