use chrono::Local;

use crate::models::submission;
use crate::pages::prelude::*;

#[post("/submissions/{uuid}/balloon-delivered")]
pub async fn post_submission_balloon_delivered(
    identity: Identity,
    pool: Data<DbPool>,
    path: Path<(String,)>,
    request: HttpRequest,
) -> PageResult {
    let logged_user = require_identity(&identity)?;
    if !logged_user.is_admin {
        return Err(PageError::Forbidden(
            "Apenas administradores podem fazer isso".into(),
        ));
    }
    let (submission_uuid,) = path.into_inner();
    let mut connection = pool.get()?;

    submission::deliver_balloon(
        &mut connection,
        submission_uuid,
        Some(Local::now().naive_utc()),
    )?;

    Ok(redirect_to_referer(
        "Tarefa fechada com sucesso".into(),
        &request,
    ))
}
