use crate::models::repository;
use crate::pages::prelude::*;
use git2::{CertificateCheckStatus, Cred, RemoteCallbacks};

#[post("/repositories/{id}/sync")]
async fn sync_repository(
    identity: Identity,
    pool: Data<DbPool>,
    path: Path<(i32,)>,
    request: HttpRequest,
) -> PageResult {
    use std::path::Path;

    let logged_user = require_identity(&identity)?;
    if !logged_user.is_admin {
        return Err(PageError::Forbidden(
            "Apenas administradores podem fazer isso".into(),
        ));
    }
    let (repository_id,) = path.into_inner();
    let mut connection = pool.get()?;
    let repository = repository::get_repository_by_id(&mut connection, repository_id)?;
    let git_repository =
        git2::Repository::open(Path::new(&format!("/data/{}/", repository.path))).unwrap();

    let mut fo = git2::FetchOptions::new();
    let mut callbacks = RemoteCallbacks::new();
    callbacks.credentials(|_url, username_from_url, _allowed_types| {
        Cred::ssh_key(
            username_from_url.unwrap(),
            None,
            Path::new("/data/id_ed25519_git"),
            None,
        )
    });
    callbacks.certificate_check(|_cert, _hostname| {
        return Ok(CertificateCheckStatus::CertificateOk);
    });
    fo.remote_callbacks(callbacks);

    git_repository
        .find_remote("origin")
        .unwrap()
        .fetch(&["main"], Some(&mut fo), None)
        .unwrap();
    let oid = git_repository
        .refname_to_id("refs/remotes/origin/main")
        .unwrap();
    git_repository
        .reset(
            &git_repository.find_object(oid, None).unwrap(),
            git2::ResetType::Hard,
            None,
        )
        .unwrap();

    Ok(redirect_to_referer(
        format!("Repositório {} atualizado com sucesso!", repository.name),
        &request,
    ))
}
